/**
 * 
 */
package de.urszeidler.eclipse.callchain.ui.wizards;

import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbench;
import org.osgi.framework.Bundle;

/**
 * @author urs
 * 
 */
public class CreateDefaultAtlFlowProject extends BasicTemplateWizard {

	private static final String GMF_TEMPLATE_PATH = "/templates/GMF_from_ECORE";
	private static final String ATL_PROJECT_NAME = "atl flow transformations";
	private static final String ATLCOPY_TEMPLATE_PATH = "/templates/mapping2atl";
	private static final String TOPCASE_DIAGRAM_TEMPLATE_PATH = "/templates/Editor_and_Diagram_from_Genmodel";
	private static final String TOPCASE_PROPERTY_TEMPLATE_PATH = "/templates/Topcased_PropertiesView";
	private AtlFlowProjectPage projectPage;
	private String createdFile;
	private Set<String> excludeFiles_gmfTemplates = new HashSet<String>();
	private IProject project;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
	 * org.eclipse.jface.viewers.IStructuredSelection)
	 */
	@Override
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		// excludeFiles_gmfTemplates.add("createGMFmapping.callchain");
		// excludeFiles_gmfTemplates.add("createGMFmapping.callchain_diagram");
		excludeFiles_gmfTemplates.add("[default] createMappin_%name%.launch");
		excludeFiles_gmfTemplates
				.add("[startpoint] bootstrapToolTemplate_%name%.launch");
		excludeFiles_gmfTemplates.add("ecore2gmfTool_custom_%name%.atl");

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.ui.wizards.BasicTemplateWizard#getModelName
	 * ()
	 */
	@Override
	protected String getModelName() {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.urszeidler.eclipse.callchain.ui.wizards.BasicTemplateWizard#
	 * getBaseTemplatePath()
	 */
	@Override
	protected String getBaseTemplatePath() {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.urszeidler.eclipse.callchain.ui.wizards.BasicTemplateWizard#
	 * getTargetContainer()
	 */
	@Override
	protected IContainer getTargetContainer() {
		return project;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.ui.wizards.BasicTemplateWizard#getModelFile
	 * ()
	 */
	@Override
	protected String getModelFile() {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.ui.wizards.BasicTemplateWizard#getProjectName
	 * ()
	 */
	@Override
	protected String getProjectName() {
		return ATL_PROJECT_NAME;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {

		final boolean addAltNature = projectPage.isAddAltNature();
		final int natureType = projectPage.getNatureType();
		final boolean canCreateTopcasedView = projectPage
				.canCreateTopcasedView();
		final boolean canCreateGMF = projectPage.canCreateGMF();
		final boolean canCreateAtlCopy = projectPage.canCreateAtlCopy();
		final boolean canCreateTopcasedDiagram = projectPage
				.canCreateTopcasedDiagram();

		if (projectPage.isPageComplete()) {

			IRunnableWithProgress op = new IRunnableWithProgress() {

				@Override
				public void run(IProgressMonitor monitor)
						throws InvocationTargetException, InterruptedException {

					if (monitor != null) {
						monitor.beginTask("creating templates", 6);
						if (monitor.isCanceled())
							return;
					}
					try {
						boolean result = true;
						createProject(monitor);
						if (monitor != null) {
							monitor.worked(1);
							if (monitor.isCanceled())
								return;
						}

						
						if (canCreateGMF) {
							result = result
									& createFilesFromTemplate(
											GMF_TEMPLATE_PATH,
											excludeFiles_gmfTemplates,
											"ecore_to_gmf",monitor);
							if (monitor != null) {
								monitor.worked(1);
								if (monitor.isCanceled())
									return;
							}

						}
						if (canCreateAtlCopy) {
							result = result
									& createFilesFromTemplate(
											ATLCOPY_TEMPLATE_PATH,
											Collections.EMPTY_SET, "atlcopy",monitor);
						}
						// if (projectPage.canCreateEcore()) {
						// result = result
						// & createFilesFromTemplate(ATLCOPY_TEMPLATE_PATH,
						// Collections.EMPTY_SET);
						// }
						if (canCreateTopcasedDiagram) {
							result = result
									& createFilesFromTemplate(
											TOPCASE_DIAGRAM_TEMPLATE_PATH,
											Collections.EMPTY_SET,
											"topcased_diagram",monitor);
						}

						if (canCreateTopcasedView) {
							result = result
									& createFilesFromTemplate(
											TOPCASE_PROPERTY_TEMPLATE_PATH,
											Collections.EMPTY_SET,
											"topcase_property_page",monitor);
						}

						if (addAltNature) {
							addATLnature(null, natureType);
						}
					} catch (Exception e) {
						// TODO: handle exception
					}
				}
			};

			try {
				getContainer().run(true, false, op);
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}
		return true;
	}

	private void createProject(IProgressMonitor monitor) throws CoreException {
		if (monitor != null) {
			monitor.subTask("creating project "+ATL_PROJECT_NAME);
			if (monitor.isCanceled())
				return;
		}

		
		IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();

		project = workspaceRoot.getProject(ATL_PROJECT_NAME);
		if (!project.exists())
			project.create(monitor);

		project.open(monitor);
	}

	private boolean createFilesFromTemplate(String templatePath,
			Set<String> excludeFiles, String targetDirectoryName, IProgressMonitor monitor)
			throws MalformedURLException, CoreException {
		Bundle extensionBundle = WizardsPlugin.getDefault().getBundle();
		IContainer target = getTargetContainer();
		IFolder folder = target.getFolder(new Path(targetDirectoryName));
		if (!folder.exists())
			folder.create(false, true, null);

		target = folder;

		Enumeration<?> entryPaths = extensionBundle.getEntryPaths(templatePath);

		String apath = buildAbsolutePath(target.getLocationURI().toURL()
				.getPath());

		initVariables("", target.getFullPath().toString(), apath,
				getModelFile());
		processPath(entryPaths, target, extensionBundle, templatePath,
				excludeFiles,monitor);

		return true;

	}

	public void addPages() {
		projectPage = new AtlFlowProjectPage(
				"Create a Atl flow example transformations");
		projectPage.setTitle("Callchain Diagrams");
		projectPage.setDescription("Select your ecore input model.");

		addPage(projectPage);

	}
}
